local IO = _G.RaidHeal.IO
local Locale = _G.RaidHeal.Locale

function IO.Print(locale, ...)
    local msg = IO.Format(locale, ...)
    if msg and msg ~= "" then IO.PrintRaw(msg) end
end

function IO.Format(locale, ...)
    if Locale.Messages[locale] then
            local msg = string.format(Locale.Messages[locale], ...)
        return msg
    else
        return ""
    end
end

function IO.LoadFile(path, silent)
    path = IO._basePath .. path

    local f, err = loadfile(path)

    if err then
        if silent then
            IO.Print("ERROR_IO_LOADFILE", path, err)
        else
            error("IO.LoadFile: Error: "..tostring(err), 2)
        end
        return nil, err
    else
        return f()
    end
end

IO.PrintRaw = SendSystemChat

function IO.DeepCopy(from, to)
    for key, value in pairs(from) do
        if type(value) == "table" then
            if not to[key] then
                to[key] = {}
            end
            IO.DeepCopy(from[key], to[key])
        else
            to[key] = value
        end
    end
end

function IO.StripCopy(copy, compareTo)
    local ret = {}
    for key, value in pairs(copy) do
        if type(value) == "function" then
            ret[key] = nil
        elseif type(key) == "string" and key:sub(1, 1) == "_" then
            ret[key] = nil
        elseif type(value) == "table" and type(compareTo[key]) == "table" then
            ret[key] = IO.StripCopy(copy[key], compareTo[key])
            if not ret[key] or ret[key] == {} then ret[key] = nil end
        elseif copy[key] == compareTo[key] then
            ret[key] = nil
        else
            ret[key] = copy[key]
        end

        if type(ret[key]) == "table" and next(ret[key]) == nil then ret[key] = nil end
    end

    return ret
end

IO.Errors = {}
IO.LastError = ""
function IO.PrintError(error)
    if not error then return end
    IO.Errors[error] = (IO.Errors[error] or 0) + 1

    if IO._printErrors and error ~= IO.LastError then
        IO.Print("ERROR_GENERAL", tostring(error))
        IO.LastError = error
    end
end

if not IO.PrintDebug then
    function IO.PrintDebug() end
end

IO.FormatTable = function(input, indent)
    local msg = ""
    if not indent then indent = "" end
    if type(input) == "nil" then return "nil" end
    if type(input) == "boolean" then return input and "true" or "false" end
    if type(input) == "string" then return "\"" .. input .. "\"" end
    if type(input) ~= "table" then return tostring(input) end

    local formatted = {}
    local innerIndent = indent .. "    "
    for key, value in pairs(input) do
        formatted[#formatted+1] = string.format("%s[%s] = %s", indent, IO.FormatTable(key, innerIndent), IO.FormatTable(value, innerIndent))
    end

    msg = string.format("{\n%s\n%s}", table.concat(formatted, ",\n"), indent)
    return msg
end

IO.FormatFuncs = {
    Numbers = {
        Simple = function(cur, max, ...)
            if not cur then return end
            if max then
                return IO.FormatFuncs.Numbers.Simple(cur - max, ...)
            end

            if cur == 0 then return "" end

            local abs = math.abs(cur)
            local format = "%s%d%s"
            local abbrev = ""

            while abs > 1000 do
                abs = abs/1000
                abbrev = abbrev .. "k"
            end

            if abs > 750 then
                local format = "%s%.1f%d"
                abs = abs/1000
                abbrev = abbrev .. "k"
            elseif abs < 100 then
                local format = "%s%.1f%s"
            end

            local sign = cur/math.abs(cur) > 0 and "" or "-"
            return string.format(format, sign, abs, abbrev)
        end
    }
}
